@* Barrier functions. The next step necessary in order to achieve our goal is to define barrier
functions. Barrier functions, as aligned before, are functionals that are bounded above if and
only if the constraint to which they relate is feasible. In our convention, a constraint is
feasible when its defining function is negative.
@s barrier int
@c /*Empty, waiting for |export| */
@ @(barrierbase.h@>=
#ifndef __MATH_BARRIER__
#define __MATH_BARRIER__ 1.0
#include <math/functionalbase.h>
#include <math/functionbase.h>
namespace math
{
    @[namespace @!barrier@]
    {
	template<class T>
	@[class base : public functional::base<T>@]
	{
	    @<Barrier function internal variables@>@;
	public:@;
	    @<Barrier function methods@>@;
	};
    }
}
#endif
@ In order to make things easier for the programmer, we allow a barrier function to be related to 
a {\it vector-valued function}. We will store pointers to the function or functional, and the
convention is that only one of them can be nonzero at any time.
@<Barrier function internal variables@>=
protected:
functional::base<T>* fctnal;
function::base<T>* fct;
@ @<Barrier function methods@>=
base(functional::base<T>* newf=0) : fctnal(newf), fct(0), has_t(false)@+{@+}
base(function::base<T>* newf) : fctnal(0), fct(newf), has_t(false)@+{@+}
void f(functional::base<T>* newf) 
{
    fctnal=newf;
    fct=0;
}
void f(function::base<T>* newf)
{
    fctnal=0;
    fct=newf;
}
virtual ~base(void)@+{@+}
@ We don't need to define the gradient and Hessian methods because we're already derived from a
class that derive them. The only thing we have to define is the behavior when dealing with
vector-valued functions: in this case, the gradient is the sum of the gradient of the vector
elements, and similarly for the Hessian and the evaluation functions. There's another important
thing that must be taken into account by evaluation methods.
@ We use barrier function in constrained optimization problems. Some algorithms don't require an
initial feasible point, that is, we don't need an~$x$ such that~$f(x)<0$. One standard way to
find a feasible point is to add a variable to the constraint so that~$f(x)<t$ is feasible, and
then minimize~$t$. These methods will then call the |addt| function of the barrier functions to
signal that we should behave as if this variable existed. From then on, until a call for |delt|,
the last component of the argument for |eval| and so on is considered to be~$t$, and the
evaluation methods must deal with it accordingly.
@<Barrier function internal variables@>=
bool has_t;
@ The |addt| function returns a value of~$t$ that satisfies~$f(x)\leq t$.
@<Barrier function methods@>=
T addt(const matrix<T,unstructured,dense>& x)
{
    T result=T(0);
    if(fctnal) result=fctnal->eval(x);
    else
    {
        matrix<T,unstructured,dense> aux;
	fct->eval(x,aux);
	for(index i=1;i<=aux.rows();++i) result+=aux(i);
    }
    has_t=true;
    return result;
}
@ @<Barrier function methods@>=
void delt(void)@+{@+has_t=false;@+}
@ Barrier functions are used in optimization problems in order to deal with constraints. To each
constraint we have an associated dual variable. This is what happens: we will want to solve the
problem $$\minimize{f_0(x)}{f_i(x)<0}$$ and we will sequentially solve $${\rm minimize}\quad \mu
f_0(x)+\sum\phi(f_i(x))$$ for various values of~$\mu$. For each of these values, the solution of
the second problem will also satisfy $\nabla f_0(x)+\sum\lambda_i\nabla f_i(x)=0$ for
some~$\lambda_i$ that depends on the particular barrier function used. But if~$x^*$ satisfies
that equality, it is a minimizer for the Lagrangian with the same~$\lambda_i$, that is,
$\lambda_i$ are feasible dual variables and can provide a lower bound on the optimal value.
It is clear that the~$\lambda_i$ will be dependent on~$\mu$, but the barrier function doesn't
know its value. What we do then is to return $\mu\lambda_i$, which is independent of~$\mu$.
@<Barrier function methods@>=
virtual T dual(const matrix<T,unstructured,dense>&)=0;
@ On other ocasious it will be useful to get the value of the dual variable times the function
itself, and it is sometimes much cheaper to compue this value (see, for example, the log barrier
function). The same above remarks with respect to~$\mu$ are valid here, that is, the function
should return~$\mu\lambda_i f(x)$.
@<Barrier function methods@>=
virtual T dual_times_f(const matrix<T,unstructured,dense>&)=0;