@** The Cholesky decomposition. The LU decomposition works for all types of square matrices.
If, however, a matrix is symmetric, then we will see that we can cut the work in half:
first, instead of decomposing~$A=LU$ we do~$A=LDU$, where~$D$ is diagonal and both~$L$
and~$U$ have unit diagonal (the matrix~$L$ is already unit diagonal, and we can make the
matrix~$U$ the same by scaling it with a diagonal matrix~$D$). Now, if~$A$ is symmetric,
then we have $L^{-1}AL^{-T}=DUL^{-T}$ is also symmetric, but this can only be true
if~$U=L^T$. Hence, if~$A$ is symmetric we can decompose it such that~$A=LL^T$, so we need to 
find only one matrix. If, in addition, $A$ is positive definite, then there is no need for
pivoting, and the resulting decomposition is called the ``Cholesky decomposition.''
@c
/* Empty, waiting for |export| */
@ @(cholesky.h@>=
#ifndef __MATH_CHOLESKY__
#define __MATH_CHOLESKY__ 1.0
#include <math/math.h>
#include <math/symmetric.h>
#include <math.h> /* for |sqrt|. */
namespace math
{@/
    namespace cholesky
    {
    @<Cholesky prototypes@>@;
    }
}
#endif
@ Another way to see that we can decompose~$A=LL^T$ if~$A$ is symmetric is by estabilishing
the equality $$ \bmatrix{ a_{11} & \alpha^T \cr \alpha & B }= \bmatrix{ \sqrt{a_{11}} & 0 \cr
\alpha/\sqrt{a_{11}} & I } \bmatrix{ 1 & 0 \cr 0 & B-\alpha\alpha^T/a_{11}} \bmatrix{
\sqrt{a_{11}} & \alpha^T/\sqrt{a_{11}} \cr 0 & I },$$ which already hints the algorithm. One
characteristic of the decomposition is that it is stable even without pivoting, and that
during the entire algorithm all diagonal elements remain positive if the matrix itself is
positive definite. The function we will define will overwrite the upper triangular part
of~$A$ with~$L^T$. We do that because now we know that a symmetric matrix structure stores
only the upper triangle elements.
@<Cholesky prototypes@>=
template<matrix_simple_template>@/
void decompose(matrix<T,structure,storage>* A)
{
    index n=A->rows();
    if(n!=A->cols()) throw error::nonsquare();
    
    for(index k=1;k<=n;++k)
    {
        if(A->get(k,k)<=0) throw error::nonpositivedef();
        @<Build $L^T$ row@>;
        @<Apply Cholesky transformation@>;
    }
}
@ @<Build $L^T$ row@>=
A->entry(k,k)=sqrt(A->get(k,k));
for(index j=k+1;j<=n;++j)
    if(!finite(A->entry(k,j)=A->get(k,j)/A->get(k,k)))
    throw error::singular();
@ @<Apply Cholesky transformation@>=
for(index i=k+1;i<=n;++i)
    for(index j=i;j<=n;++j)
        A->entry(i,j)=A->get(i,j)-A->get(k,i)*A->get(k,j);
@* Solving a linear system. The Cholesky decomposition uses half the number of flops as the
LU, and in addition there is no pivoting overhead. It is only advisable to use it to solve
linear equations with positive definite matrices. We solve a system~$AX=B$ vector by vector
of~$X$ by first solving~$Ly=b$ and then~$L^Tx=y$. As with the LU decomposition we define a
function that solves the system given a previously computed decomposition. The solution~$X$
is overwritten in matrix~$B$.
@<Cholesky prototypes@>=
template<class T, template<@[class@]> class structure_A,
template<@[class@]> class structure_B, template<@[class@]> class storage>@/
void finish(const matrix<T,structure_A,storage>& A, matrix<T,structure_B,storage>* B)
{
    index n=A.rows();
    
    for(index k=B->cols();k!=0;--k)
    {
        for(index i=1;i<=n;++i) /* Solve $Ly=B(:,k)$. */
        {
            T inner=0;
            for(index j=1;j<i;++j)
            /* Here we have to remember that we overwrote only the upper triangular part
               of~$A$, so that now we have to get the element~$A(j,i)$ instead of~$A(i,j)$. */
                inner += A(j,i)*B->get(j,k);
            if(!finite(B->entry(i,k)=(B->get(i,k)-inner)/A(i,i)))
		throw error::singular();
        }
        for(index i=n;i>=1;--i) /* Solve $L^Tx=y$. */
        {
            T inner=0;
            for(index j=i+1;j<=n;++j)
                inner += A(i,j)*B->get(j,k);
            if(!finite(B->entry(i,k)=(B->get(i,k)-inner)/A(i,i)))
		throw error::singular();
        }
    }
}
@ Finally, we provide an interface for the linear solver. The big warning is that the~$A$ matrix
will be overwritten too, not only~$B$!!!
@<Cholesky prototypes@>=
template<class T, template<@[class@]> class structure_A,
    template<@[class@]> class structure_B,template<@[class@]> class storage>@/
matrix<T,structure_B,storage>&
solve(matrix<T,structure_A,storage>* A, matrix<T,structure_B,storage>* B)
{
    decompose(A);
    finish(*A,B);
    return *B;
}
@* Interfacing with \lapack. Again, the routines we just defined can compute the Cholesky
decomposition of any matrix. If you have \lapack{} installed, however, you can take
advantage of years and years of laborious code optimization -- why not use it?
@<Cholesky prototypes@>=
#include <math/private/fortran.h>
#ifdef HAVE_LIBLAPACK
@<Cholesky lapack interface@>@;
#endif
@ We begin with the double precision symmetric matrix case. The others are very similar, we
just have to replace the function call.
@<Cholesky lapack interface@>=
template<>@/
void decompose(matrix<fortran::double_precision,symmetric,dense>* A)
{
    @<Prepare for lapack Cholesky@>@;    
    dpptrf(&mode,&n,A->storg()->memory(),&status);
    @<Check lapack Cholesky errors@>@;
}
@ @<Prepare for lapack Cholesky@>=
fortran::integer n=A->rows();
fortran::integer m=A->cols();
fortran::integer status=0;
fortran::character mode='U';

if(n!=m) throw error::nonsquare();
@ @<Check lapack Cholesky errors@>=
if(status>0) throw error::nonpositivedef();
if(status<0) throw error::generic();
@ As said before, the rest of the routines is essentially the same, except that for
unstructured matrices we need to pass also the number of columns to the \lapack{} routines.
@<Cholesky lapack interface@>=
template<>@/
void decompose(matrix<fortran::real,symmetric,dense>* A)
{
    @<Prepare for lapack Cholesky@>@;    
    spptrf(&mode,&n,A->storg()->memory(),&status);
    @<Check lapack Cholesky errors@>@;
}
@ We begin with the symmetric matrix cases.
@<Cholesky lapack interface@>=
template<>@/
void decompose(matrix<fortran::double_precision,unstructured,dense>* A)
{
    @<Prepare for lapack Cholesky@>@;    
    dpotrf(&mode,&n,A->storg()->memory(),&m,&status);
    @<Check lapack Cholesky errors@>@;
}
@ @<Cholesky lapack interface@>=
template<>@/
void decompose(matrix<fortran::real,unstructured,dense>* A)
{
    @<Prepare for lapack Cholesky@>@;    
    spotrf(&mode,&n,A->storg()->memory(),&m,&status);
    @<Check lapack Cholesky errors@>@;
}
