@** File streams: input. Files are the most convenient interface with other programs. I
chose the \matlab{} file format for simple interface with a widely used program. Since
\matlab's version~4 file format is public we will use it against the new format, which is
proprietary and secret and sucks.
@c
#include "fstream.h"
@ @(fstream.h@>=
#ifndef __MATH_FSTREAM__
#define __MATH_FSTREAM__ 1.0
@<Include files |fstream|@>@;
#include <math/math.h>
namespace math
{
    @<|fstream| structures@>@;
    @<|fstream| declarations@>@;
}
#endif
@ We will first define the input stream, so that you will be able to load matrices from a
\matlab{} file. Almost all the work is already done by the standard library \&{ifstream}
class, the only thing we need to do is to specialize it a little bit. We will want, for
example, to find a matrix by name.
@s ifstream int
@<|fstream| declarations@>=
class ifstream : public std::ifstream
{
public:@;
    @<|ifstream| methods@>@;
};
@ @<Include files |fstream|@>=
#include <fstream>
@ The first thing to do is to overload some constructors for compatibility with the standard
|ifstream| class.
@<|ifstream| methods@>=
ifstream() : std::ifstream()@+ {}
ifstream(int fd) : std::ifstream(fd)@+ {}
@ Now to the opening of files. A matrix file is binary, so we change the default |ifstream|
mode on both the constructor and the |open| method accordingly.
@<|ifstream| methods@>=
ifstream(const char* name, int mode= ios::in | ios::binary , int prot=0664) :
std::ifstream(name, mode, prot) {}
@ @<|ifstream| methods@>=@!@:ifstream}{open@>
void open(const char* name, int mode=ios::in|ios::binary,int prot=0664)
{ std::ifstream::open(name,mode,prot); }
@ Unfortunately, the binary format of my lybrary does not work for |long| or |double|, that
is, I cannot use the operator |>>| to get these value types from a file. But we can fix it
very easily by defining new operators. The trick, in both cases, is to declare an union of
|unsigned char| and |long| (or |double|) and read |unsigned char|s from the file. Since we
have the desired type in the union, what we are actually doing is filling in the value. We
will only overload the |>>| operator for the |long| case, because it will be convenient when
reading matrices headers. For the |double| case it is simpler to do the trick when reading
the matrix data itself.
@<|ifstream| methods@>=
ifstream& operator>>(long&);
@ @c
math::ifstream& math::ifstream::operator>> (long& dest)
{
    union
    {
        long Long;
        unsigned char Char[sizeof(long)];
    }
    tricky;
    
    for(int i=0;i!=sizeof(long);this->get(tricky.Char[i++]));
    
    dest=tricky.Long;
    return *this;
}
@ Now we define a method that will read a matrix header. In \matlab{} files, a matrix header
consists of a sequence of fields that we will read in a structure as described below.
@<|fstream| structures@>=
struct fheader
{
    long type; /* Type of the object. */
    long rows; /* Number of rows. */
    long cols; /* Number of cols. */
    bool iflag; /* |true| if matrix has imaginary part. */
    string name; /* Matrix name. */
};
@ @<Include files |fstream|@>=
#include <string>
@ The actual data on a \matlab{} file is somewhat different. The three first fields from
|fheader| are the same. The |iflag| field is a |long| on the original file, and after it
there is a |long| field with the number of characters in the name (including the terminating
|'\0'|) followed by the matrix name. If we encounter an error while parsing the header we
return, instead of throwing an error, since this method is not intended for end users.
@<|ifstream| methods@>=
void parse_header(fheader& header);
@ @c
void @[math::ifstream::parse_header@](math::fheader& header)
{
    long name_length;
    long file_flag;
    *this >>header.type>>header.rows>>header.cols>>file_flag>>name_length;
    if(rdstate()!=goodbit) return;
    header.iflag=bool(file_flag);

    header.name.assign(name_length,0); /* Reserve space for matrix name. */
    for(int i=0;i!=int(name_length);this->get(header.name[i++]));
    
}
@ We are now in position to load a matrix. In a \matlab{} file the matrix data is stored by
columns, in two separate blocks for real and imaginary parts. If the matrix does not have an
imaginary part there is only the real block in the file. There is only one complication: if
we are to define a single method, then we must call, in case of a complex matrix, the method
|set| with a complex argument. Even if this method is not called it must be compiled. Now,
if the matrix is of type, say, |double|, then there will be a compiler error, because there
is no conversion from |complex| to |double|. The solution to this problem is to define two
functions, specializing for complex matrices. First we will define the method for
non-complex matrices. In this case we simply disconsider the eventual imaginary part of the
file matrix.
@<|ifstream| methods@>=
template<matrix_simple_template>@/
ifstream& operator>>(matrix<T,structure,storage>& dest)
@ Until the compiler accepts |export| we are stuck with the inline
method.
@<|ifstream| methods@>=
{
    math::fheader header;
    parse_header(header);

    if(rdstate()!=goodbit) throw math::error::filerr();

    dest.init(header.rows,header.cols);
    for(int ipart=0;ipart<=header.iflag;ipart++)
        for(int j=1;j<=header.cols;j++)
            for(int i=1;i<=header.rows;i++)
            {
                double number;
                @<Get |number| from file, using...@>@;
                if(!ipart)
                    dest.set(i,j,number);
            }
    return *this;
}
@ @<Get |number| from file, using the same tricky method as for |long|s@>=
union 
{
    double Double;
    unsigned char Char[sizeof(double)];
} tricky;
for(int k=0;k!=sizeof(double);this->get(tricky.Char[k++]));
number=tricky.Double;
@ Next we do the same thing for |complex| matrices. In this case there is nothing to worry
about: if the file matrix does not have an imaginary part, the matrix itself will have the
imaginary part zeroed.
@<|ifstream| methods@>=
template<matrix_simple_template>@/
ifstream& operator>>(matrix<complex<T>,structure,storage>& dest)
@ Until the compiler accepts |export| we are stuck with the inline
method.
@<|ifstream| methods@>=
{
    math::fheader header;
    parse_header(header);

    if(rdstate()!=goodbit) throw math::error::filerr();

    dest.init(header.rows,header.cols);
    for(int ipart=0;ipart<=header.iflag;ipart++)
        for(int j=1;j<=header.cols;j++)
            for(int i=1;i<=header.rows;i++)
            {
                double number;
                @<Get |number| from file, using...@>@;
                if(!ipart)
                    dest.set(i,j,number);
                else
                    dest.set(i,j,dest(i,j)+number*@[std::complex<double>(0,1)@]);
            }
    return *this;
}
@ @<Include files |fstream|@>=
#include <complex>
@ At this point we are able to load a matrix. Sometimes, however, we will need to skip
matrices (to load the second one, for example) or to find matrices by name. We begin by
defining a method to skip matrices. First we parse the current matrix header and then skip
the data using a |skip_data| method.
@<|ifstream| methods@>=@!@:ifstream}{skip@>
void skip(int num_matrices);
@ @c
void @[math::ifstream::skip@](int num_matrices)
{
    math::fheader header;
    while(num_matrices--)
    {
        parse_header(header);
        if(rdstate()!=goodbit)@/
	    throw math::error::generic("Too many matrices to skip.");
        skip_data(header);
    }
}
@ Skipping the data is a simple matter of reading the necessary amount of bytes from the
body of the matrix.
@<|ifstream| methods@>=@!@:ifstream}{skip_data@>
void skip_data(const fheader& header);
@ @c
void @[math::ifstream::skip_data@](const math::fheader& header)
{
    long data_size = header.cols * header.rows * sizeof(double);
    if(header.iflag) data_size *= 2;
    for(unsigned char garbage; data_size; data_size--)
        this->get(garbage);
}
@ With the above definitions we can jump to an arbitrary matrix on the file. The first
matrix has position~1.
@<|ifstream| methods@>=@!@:ifstream}{skipto@>
void skipto(const int position);
@ @c
void @[math::ifstream::skipto@](int position)
{
    seekg(0);
    if(--position>0) skip(position);
}
@ Now to matrix names. What we do when we want to skip to a certain matrix name is to first
go to the beginning of the file and search for the matrix with the correct name. If we
succeed, then the file will be {\em after} the header, so that the input operator cannot be
used. The solution is to rewind the file again and skip to the correct matrix.
@<|ifstream| methods@>=@!@:ifstream}{skipto@>
void skipto(const char* matrix_name);
@ @c
void @[math::ifstream::skipto@](const char* matrix_name)
{
    int position=0;
    seekg(0);
    @<Search for correct matrix name@>@;
    skipto(position);
}
@ @<Search for correct matrix name@>=
math::fheader header;
header.name="";
while(strcmp(header.name.c_str(),matrix_name))
{
    if(position) skip_data(header); /* Skip data of previous matrix */
    parse_header(header);
    if(rdstate()!=goodbit)@/
	throw math::error::generic("No matrix with supplied name in the file.");
    position++;
}
@** File streams: output. The input part is complete, so let us go on the next part. Again,
the biggest job is already done by the standard library \&{ofstream} class, we just need to
specialize it a little. The problem now is that we must give matrices a name before writing
them to a file. We will do that by keeping a private string for the matrix name in the class
and providing a means to modify it.
@s ofstream int
@<|fstream| declarations@>=
class @!ofstream : public std::ofstream
{
    string matrix_name;
public:@;
    @<|ofstream| methods@>@;
};
@ We overload some constructors for compatibility with the standard \&{ofstream} library.
@<|ofstream| methods@>=
ofstream() : std::ofstream()@+ {}
ofstream(int fd) : std::ofstream(fd)@+ {}
@ Now to the opening of files. A matrix file is binary, so we change the default |ofstream|
mode on both the constructor and the |open| method accordingly.
@<|ofstream| methods@>=
ofstream(const char* name, int mode= ios::out | ios::binary , int prot=0664) :
std::ofstream(name, mode, prot) {}
@ @<|ofstream| methods@>=@!@:ofstream}{open@>
void open(const char* name, int mode=ios::in|ios::binary,int prot=0664)
{ std::ofstream::open(name,mode,prot); }
@ Now to the writing algorithms: since we normally write matrices in sequence, providing a
method like the |ifstream|'s |skipto| is unhandy and cumbersome to use. The solution is to
``write the matrix name'' to the stream before writing the matrix itself. This operation
will assign the matrix name field of the header to the specified name.
@<|ofstream| methods@>=
ofstream& operator<<(const char* name);
@ @c
math::ofstream& math::ofstream::operator<<(const char* name)
{
    matrix_name=name;
    return *this;
}
@ Like with the input stream case, we define a method to output |long|s. The trick used is
exactly the same.
@<|ofstream| methods@>=
ofstream& operator<<(const long&);
@ @c
math::ofstream& math::ofstream::operator<<(const long& source)
{
    union
    {   long Long;
        unsigned char Char[sizeof(long)];
    }
    tricky;
    
    tricky.Long=source;
    for(int i=0;i!=sizeof(long);this->put(tricky.Char[i++]));
    return *this;
}
@ We are now ready to write matrices. We have here the same problem with |complex| types as
we had in the input stream case, and we adopt the same specialization solution.
@<|ofstream| methods@>=
template<matrix_simple_template>@/
ofstream& operator<<(const matrix<T,structure,storage> &source)
@ Waiting for |export|...
@<|ofstream| methods@>=
{
    long iflag=0;
    @<Write matrix header@>@;
    
    for(math::index j=1;j<=source.cols();++j)
        for(math::index i=1;i<=source.rows();++i)
        {
            double number;
            number=double(source(i,j));
            @<Write |number| to file, using...@>@;
        }

    return *this;
}
@ @<Write matrix header@>=
*this << long(0)<<long(source.rows())<<long(source.cols()) << iflag << long(matrix_name.size()+1);
for(unsigned int i=0;i<=matrix_name.size();this->put(matrix_name.c_str()[i++]));
@ @<Write |number| to file, using the same tricky mehtod as for |long|s@>=
union
{
    double Double;
    unsigned char Char[sizeof(double)];
} tricky;
tricky.Double=number;
for(int k=0;k!=sizeof(double);this->put(tricky.Char[k++]));
@ Now to the complex specialization.
@<|ofstream| methods@>=
template<matrix_simple_template>@/
ofstream& operator<<(const matrix<complex<T>,structure,storage> &source)
@ Waiting for |export|...
@<|ofstream| methods@>=
{
    long iflag=1;
    @<Write matrix header@>@;
    
    for(int ipart=0;ipart<=1;++ipart)
        for(math::index j=1;j<=source.cols();++j)
            for(math::index i=1;i<=source.rows();++i)
            {
                double number;
                complex<double> aux;
                aux=@[complex<double> (source(i,j))@];
                if(!ipart) number=aux.real();
                else number=aux.imag();
                @<Write |number| to file, using...@>@;
            }

    return *this;
}
@ With the end of the file streams classes we finished the main body of the library. With
the definitions so far provided, you are able to do everything you want, including
interfacing with \matlab. Of course, what we have is too basic for complex algorithms, so
our task now is to provide common functions, such as various types of matrix decompositions,
solution of systems, and functions like determinant, inverse and so on. The point is that
the main job is done, that is, the definition of the structures and methods necessary to
develop algorithms. From now on there is really nothing really new, no impacting decisions
to be made. The fun is not gone, but instead is replaced by the fun of using the power of
the \&{matrix} class to write beautiful code.
