@** Basic definitions. The natural % -*- mode: cweb; mode: fold; -*- 
place to start is with the basic definitions, which are
the the \&{index} and the \&{error} types: together, they provide us with the necessary
tools to begin the longer \&{matrix} definition. These basic definitions, along with the
matrix definition, are declared in the \.{math.h} file. As customary, the definitions are
stated in the header file, and some bigger function bodies are defined in \.{math.cc}. As
will happen with all \math{} classes and functions, these definitions are declared inside
the \&{math} namespace. The next three sections follow a pattern on this document: first we
define the beginning of the source and header files, and then proceed with the code.
@c
#include "math.h"
@<Big definitions@>@;
@ @(math.h@>=
#ifndef __MATH__
#define __MATH__ 1.0
@<Include files |math|@>@;
@h
namespace math
{
    @<Basic definitions@>@;
    @<Element definition@>@;
    @<Structure definition@>@;
    @<Storage definition@>@;
    @<Matrix definition@>@;
    @<Submatrix definition@>@;
    @<Basic algebraic operations@>@;
    @<Specializations@>@;
    @<|export|-waiting big definitions@>@;
}
#endif
@ The first thing we do is to define the C type that will be used for indexing elements of
matrices. We define it as an |unsigned int|, which means, for one thing, that you can not
define behaviors based on an \&{index} being negative. Since the entire library is based on
the $(1,1)$ origin default, you can always use a zero value as an indicator. Note that an
\&{index} definition already exists -- it is defined in the standard library's |string|
class. Therefore, a safe use of the type will be \PB{\&{math}\DC\&{index}}, even when you
declare that you are using namespace |math|.
@<Basic definitions@>=
typedef unsigned int @!index;
@q Error types @>
@ In order to be able to differentiate \math{} errors from others, we will define a some
classes under the namespace \PB{\&{math}\DC\&{error}} which we will use to signal anomalies.
Our objective here is to provide a consistent and flexible means of passing errors from the
library. The first generic error class will contain a string that can be used to describe
the cause of the errors. Common errors can be derived from this class.
@<Basic definitions@>= 
namespace error
{@/
class generic {
    string theMessage;
public:@;
    @<Generic error class methods@>@;
    virtual ~generic()@+ {} /* A base class it is. */
};
@<Predefined error types@>@;
}
@ @<Include files |math|@>=
#include <string>
@ The |generic| error class is intended to be used in |try|-|catch| mechanisms, so when
something goes wrong you simply |throw| an error. In this way, it doesn't make sense to
declare an |error| variable and update it during the program. Hence, the only provided way
to modify an |error| is at the time of construction, and we will then need the appropriate
constructors: the two defined below are used when the error we want to signal is not
predefined. In that case, we create an unknown error with or without an explaining message.
@<Generic error class methods@>=
generic(void) @[:@] theMessage("unknown") @+ {}
generic(const char* msg) @[:@] theMessage(msg) @+ {}
@ Of course, you may want to check out the message that was passed. We provide one method to
retreive the friendly error message. Depending on the error type we could even not consider
it an error (see, for example, the |det| function).
@<Generic error class methods@>=
@!@:generic}{message@>
const string& message(void) const@+ {@+ return theMessage;@+ }
@ Next we deal with predefined errors. What we do is to derive some classes that implement
common errors.
@<Predefined error types@>=
class singular : public generic
{ public: singular():generic("Matrix is singular to working precision.")@+ {}@+ };@;
class filerr : public generic
{ public: filerr():generic("Generic file error.")@+ {}@+ };@;
class infeasible : public generic
{ public: infeasible():generic("Problem is infeasible.")@+ {}@+ };@;
class nonsquare : public generic
{ public: nonsquare():generic("Matrix should be square")@+ {}@+ };@;
class nonpositivedef : public generic
{ public: nonpositivedef():generic("Matrix should be positive definite.")@+ {}@+ };@;
class dimension : public generic
{ public: dimension():generic("Wrong matrix dimensions.")@+ {}@+ };@;
class notimplemented : public generic
{ public: notimplemented():generic("Feature not yet implemented.")@+ {}@+ };@;
class maxiterations : public generic
{ public: maxiterations():generic("Maximum number of iterations reached.")@+ {}@+ };@;
class unboundedbelow : public generic
{ public: unboundedbelow():generic("Problem is unbounded below.")@+ {}@+ };@;
class domain : public generic
{ public: domain():generic("Domain violation.")@+ {}@+ };@;
class rankdefficient : public generic
{ public: rankdefficient():generic("Matrix is rank defficient.")@+{}@+};@;
@q@>
@q Matrix basics @>
@** Matrix basics. Let us now begin the fun part. A matrix is a template on the type of its
elements, its structure and its storage. Being that way, a matrix can be symmetric and
sparse at the same time. The matrix is, then, a container for a storage of elements that are
to be used according to some structure. The {\em function} of a matrix consists in
coordinating the use of its storage by asking the structure what to do. In what follows we
will define this protocol as necessity arrives by using the standard \&{dense} storage and
\&{unstructured} structure.
@<Storage definition@>=
template<class T>@;
class dense
{
    typedef T element_type;
    @<Dense storage internal variables@>@;
public:@;
    @<Dense storage methods@>@;
};
@ @<Structure definition@>=
template<class T>@;
class unstructured
{
    typedef T element_type;
public:@;
    @<Unstructured structure methods@>@;
};
@ Now to the user's matrix type. The first thing we need to realize is that we will
inevitably need to return matrices from some functions (it suffices to think of a function
to return the identity matrix). In order to minimize the overhead in passing matrices
around, we separate the matrix {\em representation} from the matrix {\em handle}. The
\&{matrix} type is, then, a handle to a representation that can be shared among many
matrices. An assigment of the type $A=B$ will be equivalent to ``matrix $A$ will share
matrix $B$ representation.'' Of course, we need to be careful to not modify matrices we
don't want to. For example, $A(1,1)=1$ can not modify $B$. As we proceed we will take care
of all these cases.
@d matrix_simple_template @/class T=double,@/
                          template<@[class@]> class structure=unstructured,@/
			  template<@[class@]> class storage=dense
@<Matrix definition@>=
@/
@<Matrix representation definition@>@;
template<matrix_simple_template>@;
class matrix
{
public:@;
    @<Matrix internal types@>@;
private:@;
    @<Matrix internal variables@>@;
public:@;
    @<Matrix methods@>@;
};
@ As aligned before, a matrix is simply a handle to a matrix representation. It is the
representation who holds the storage and has the structure information. Hence, a
representation is a template of all of them.
@<Matrix representation definition@>=
template<matrix_simple_template>@;
class representation
{
    typedef T element_type;
    @<Matrix representation internal variables@>@;
public:@;
    @<Matrix representation methods@>@;
};
@ A matrix knows its representation through a pointer to a it. In this way a single
representation can be shared among various matrices. This also means that it is the matrix
itself who must coordinate creation of new representations and destruction of them, as we
shall see later.
@<Matrix internal types@>=
typedef representation<T,structure,storage> @!rep_type;
@ @<Matrix internal variables@>=
rep_type *theRepresentation;
@ Before going on, we note that we must have a means to obtain the type of elements,
structure and storage of a matrix. Since a matrix is a template, we provide a means of
accessing the type of the matrix as internal types in the same way as the Standard Template
Library does.
@<Matrix internal types@>=
typedef T element_type;
typedef storage<T> storage_type;
typedef structure<T> structure_type;
typedef matrix<T,structure,storage> matrix_type;
@ Now back to business. Since the representation is the owner of the storage, it is the
representation who holds the size of the matrix, not the |matrix| type.
@<Matrix representation internal variables@>=
index num_rows,num_cols;
@ As will happen often, the representation provides methods to access the information about
the matrix, and the |matrix| type provides the interface to them. Often the representation
methods don't perform any error checking, since the user will in the end use only the
|matrix| type. The definition of two types of methods (one |const| and one not) is necessary for
specializations (for example, the |reshape| function will modify the number of rows directly).
@<Matrix representation methods@>= @!@:representation}{rows@>
@!@:representation}{cols@>
index& rows(void) @+{@+ return num_rows;@+}
index& cols(void) @+{@+ return num_cols;@+}
index rows(void) const @+{@+ return num_rows;@+}
index cols(void) const @+{@+ return num_cols;@+}
@ @<Matrix methods@>=
@!@:matrix}{cols@>
@!@:matrix}{rows@>
index rows(void) const@+
{@+ return (theRepresentation?theRepresentation->rows():0);@+}@:representation}{rows@>
index cols(void) const@+
{@+ return (theRepresentation?theRepresentation->cols():0);@+ }@:representation}{cols@>
@ Also, as aligned before, it is the representation who has the storage and structure
information. It will be the job of the representation to use them appropriately.
@<Matrix representation internal variables@>=
storage<T> * theStorage;
structure<T> * theStructure;
@q@>
@q Creating and copying @>
@** Creating and copying a matrix. We can now begin to handle the ways in which a
representation is handled by the matrix, and we will start by the ways a matrix is created.
First, we need to know how many matrices are sharing a single representation. Obviously,
this information can only be kept by the representation itself, and the matrix must be able
to retreive this information.
@<Matrix representation internal variables@>=
int num_instances;
@ When a matrix is emptied or some operation modifies it, it is often the case that a matrix
will need to create a new representation. On the other hand, copying matrices simply means
incrementing the number of shared instances of some representation. Since it is the matrix
who deals with this protocol, we return a reference to the number of instances so that a
matrix can modify it itself.
@<Matrix representation methods@>=@!@:representation}{instances@>
inline int& instances(void)@+
{@+
    return num_instances;@+
}
@ Let us then enable a matrix to be created. The default constructor should create an empty
matrix, and the dimension constructor creates a fresh representation.
@<Matrix methods@>=
matrix(void) @[:@] theRepresentation(0) @+ {}
@ @<Matrix methods@>=
matrix(const index rows, const index cols)@+
{@+
#ifdef __MATH_INSANE_DEBUG__
    cout << "[math]: matrix(" << rows << ',' << cols << ") oldrep=" << theRepresentation;
#endif
    theRepresentation = new rep_type(rows,cols);@+
#ifdef __MATH_INSANE_DEBUG__
    cout << " newrep=" << theRepresentation << '\n';
#endif
}
@ A representation is never empty. It is always created with dimensions, in the way you just
saw. In order to really forbid the empty representation creation we define a default
constructor which trhows an error.
@<Matrix representation methods@>=
representation(void)
{
    throw error::generic("Cannot instantiate empty matrix representation!");
}
@ It is the job of the representation constructor to create the structure and the storage.
By defining the representation constructor we begin, then, to definite the communication
protocol between storages and structures. We will explain the protocol by defining the
|dense| and |unstructured| classes.
@<Matrix representation methods@>=
representation(index rows, index cols) @[:@] 
    num_rows(rows), num_cols(cols), num_instances(1)
{
    theStructure = new@,@, @[structure<element_type>@](&rows,&cols);
    theStorage = new@,@, @[storage<element_type>@](rows,cols);
}
@ The first part of the protocol consists in creating a matrix with enough storage for its
elements. A symmetric matrix, for example, does not need to store $\hbox{\it rows}* \hbox{\it
cols}$ elements. Hence we first create the structure, and rely on the constructor to modify its
arguments so that the a posteriori storage creation works accordingly. For the unstructured type,
however, we don't need to modify the arguments.
@<Unstructured structure methods@>=
unstructured(index*,index*)@+{}
@ Having now the correct dimensions of the matrix we can create the storage. Let's do it for
the dense case. A dense matrix is stored as a vector in |elements| by columns, that is, the
first elements are from the first column and so on. The index to the first element of a
column is indexed in~|data|. Later on this can appear to be unintuitive, but this scheme
saves memory for vectors, when there is only one element in~|data|, and also it will facilitate
the build of a \lapack{} interface. We will also allow the
storage to have storage for more rows and columns than what is needed. This will be very useful
when resizing.
@<Dense storage internal variables@>=
element_type *elements;
element_type **data;
index num_rows, num_cols;
index max_rows, max_cols;
@ We define, as usual, a default constructor to zero everything.
@<Dense storage methods@>=
dense()@[:@] elements(0),data(0),num_rows(0),num_cols(0),max_rows(0),max_cols(0)@+{}
@ Now to the useful dense constructor. We set the initial array values to zero, and we use
|memset| for doing this. One could argue that we should do a loop and assign |element_type(0)| to
each element, but let's get real... Since the task of initializing the data is also useful
outside the constructor, we define an |init| method to be used for initialization and call it
from the constructor.
@<Dense storage methods@>=
dense(const index &rows, const index &cols):elements(0), data(0)
{
    init(rows,cols);@:dense}{init@>
    memset(elements,0,num_rows*num_cols*sizeof(element_type));
}
@ @<Include files |math|@>=
#include <string.h> /* For |memset| (and |memcpy|). */
@ The |init| function allocates space and initialize the data pointers, but leaves the data
contents unchanged. The method will be constructed so that it can be used to resize the data if
the number of elements remains the same (if they are not, you'll have segfault somewhere later).
The trick in this method is that the vectors are initialized in a way such that the $(1,1)$
origin standard works, that is, |data[1][1]| is the first element of the matrix. The only catch
is that, due to the way the data is stored, the element~$A(i,j)$ is stored in~|data[j][i]|.
@<Dense storage methods@>= @!@:dense}{init@>
void init(const index &rows, const index &cols);
@ @<|export|-waiting...@>=
template<class T>
void dense<T>::init(const index& rows, const index& cols)
{
    num_rows=max_rows=rows;
    num_cols=max_cols=cols;
    if(!elements) elements=new element_type[rows*cols];
    if(data) delete []++data;
    data=new element_type*[cols];
    for(index i=0;i<cols;++i) data[i]=elements+i*rows-1;
    data--;
}
@ Voil\`a! We are now able to create an empty matrix and a matrix filled with zeros.
Sometimes, however, it is useful to create a matrix filled with some specified value. We
provide a method to fill a matrix with a specified value and a constructor to do the same thing.
@<Matrix methods@>=@!@:matrix}{fillwith@>
void fillwith(const element_type& value);
@ @<|export|-waiting...@>=
template<matrix_simple_template>
void matrix<T,structure,storage>::fillwith(const element_type& value)
{
    for(index i=1; i<=rows(); i++)
        for(index j=1; j<=cols(); j++)
    theRepresentation->set(i,j,value);@:representation}{set@>
}
@ @<Matrix methods@>=
matrix(const index rows, const index cols, const element_type& value)
{
#ifdef __MATH_INSANE_DEBUG__
cout<<"[math]: matrix("<<rows<<','<<cols<<','<<value<< ") oldrep=" << theRepresentation;
#endif
    theRepresentation = new rep_type(rows,cols);
    fillwith(value);@:matrix}{fillwith@>
#ifdef __MATH_INSANE_DEBUG__
    cout << " newrep=" << theRepresentation << '\n';
#endif
}
@ Next we define the copy constructor. As noted before, the only thing
we need to do is to share the representation and update the number of
shared instances.
@<Matrix methods@>=
matrix(const matrix_type& source)
{
    if(source.theRepresentation)
    source.theRepresentation->instances()++;@:representation}{instances@>
    theRepresentation = source.theRepresentation;
}
@ Now we have all the basic constructors we need. The next step is to define the copy
operations, which are very similar. In order to be able to do that we define a method that
reinitializes a matrix to a given size. This method is like a constructor, but it takes into
account the fact that the matrix can already have a representation. If it has one and its
not shared and it has the same dimensions we don't need to do anything. In the other case we
need to create a new representation anyway, taking care of the old one (the deletion of a
representation is the topic of the next section, but the details are not necessary here).
@<Matrix methods@>=
@!@:matrix}{init@>
@:matrix}{cols@>@:matrix}{rows@>@:representation}{instances@>
void init(const index num_rows,const index num_cols);
@ @<|export|-waiting...@>=
template<matrix_simple_template>
void matrix<T,structure,storage>::init(const index num_rows, const index num_cols)
{
    if(theRepresentation && cols()==num_cols && rows()==num_rows
       && theRepresentation->instances()==1)@/
    return;
    if(theRepresentation && --theRepresentation->instances()==0)
        delete theRepresentation;
    theRepresentation=new rep_type(num_rows,num_cols);
}
@ We are now able to define the assignment operator.  In our context the task is a simple matter
of using the same representation. Sometimes, however, this is not what we want: we want the
contents copied and a new representation created. This normally occurs in numerical code where all
matrices are modified almost instantly anyway. We provide a way to control the behavior of the
assignment operator through the global |fast_assignment|. If |fast_assignment| is |true|, then the
assignment operator uses the faster but memorywise expensive |copyfrom| method, to be defined
later.
@<Basic definitions@>=
extern bool fast_assignment;
@ @c
bool math::fast_assignment=false;
@ Now to the method itself. In case |fast_assignment| is false, we share the representation, the
only catch being that we need to avoid confusions when making ridiculous things like~$A=A$.
@<Matrix methods@>=
@!@:matrix}{$\K$@>
matrix_type& operator=(const matrix_type& source);
@ @<|export|-waiting...@>=
template<matrix_simple_template>
matrix<T,structure,storage>::matrix_type& 
matrix<T,structure,storage>::operator=(const matrix_type& source)
{
    if(fast_assignment) return copyfrom(source);
    if(source.theRepresentation)
	source.theRepresentation->instances()++;@:representation}{instances@>
    if(theRepresentation && --theRepresentation->instances()==0) delete theRepresentation;
    theRepresentation = source.theRepresentation;
    return *this;
}
@ We may also want to perform assignments with different types of source matrices. In that case
the only way is to copy all elements, one by one. This method requires the operator |()|, which
retreives an element from a matrix, and the methods |set|, which assigns a value to a particular
element. The details are irrelevant for now.
@d matrix_template(M) @/class @[T##M@],@/
                      template<@[class@]> class @[STR##M@],@/
		      template<@[class@]> class @[STO##M@]
@<Matrix methods@>=
@!@:matrix}{$\K$@>
template<matrix_template(A)>@;
matrix_type& operator=(const matrix<TA,STRA,STOA>& source)
{
    init(source.rows(),source.cols());@:matrix}{init@>
    index i,j;
    for(i=1;i<=rows();++i)@:matrix}{rows@>
        for(j=1;j<=cols();++j)@:matrix}{cols@>
            set(i,j,source(i,j));@:matrix}{set@>
    return *this;
}
@ @<Matrix methods@>=
template<matrix_template(A)>@;
matrix(const matrix<TA,STRA,STOA>& source)@[:@]theRepresentation(0)
{@+ *this = source;@+ }
@ On ocasion we may want to copy a matrix without sharing the representation (for example, when
we know the matrix will be modified right away so that a new representation will be created
anyway if we share the instance). So here's what we do: if we don't have a representation yet we
simply create a new one. If we do have one and it has the same dimensions (we know it has the same
type), we assign a copy of the old one to it (this can be very fast). If none of this happens we
behave just like the assignment operator, except we always will create a new representation.
@<Matrix methods@>=@!@:matrix}{copyfrom@>
matrix_type& copyfrom(const matrix_type& source)
{
    if(theRepresentation==source.theRepresentation) return *this;
    if(theRepresentation && theRepresentation->instances()==1 &&
       rows()==source.rows() && cols()==source.cols()) @:matrix}{rows@>@:matrix}{cols@>
    {
	*theRepresentation= *(source.theRepresentation);
	return *this;
    }
    if(theRepresentation && --theRepresentation->instances()==0)@:representation}{instances@>
	delete theRepresentation;
    theRepresentation=new rep_type(*(source.theRepresentation));
    return *this;
}
@ In order to define the |copyfrom| method we created a representation based on an existing one.
We will need to do the same thing when setting elements. For this task we will define a copy
representation constructor, which in turn will require copy constructors for both the dense and
unstructured classes.
@<Matrix representation methods@>=
representation(const representation &source)
{
    theStructure=new@+ structure<T>(*(source.theStructure));
    theStorage=new@+ storage<T>(*(source.theStorage));
    num_instances=1;
    num_rows=source.rows();@:representation}{rows@>
    num_cols=source.cols();@:representation}{cols@>
}
@ @<Dense storage methods@>=
dense(const dense & source):elements(0),data(0)
{
    init(source.num_rows,source.num_cols);
    memcpy(elements,source.elements,num_rows*num_cols*sizeof(element_type));
}
@ @<Unstructured structure methods@>=
unstructured(const unstructured &)@+ {@+}
@ We also used assignment operators in |copyfrom|. In order to simplify things, we only allow
assignment operators to be called when the sources have the same dimensions. This may change in
the future, but for now it's good enough (since the user will never use these operators directly).
@<Matrix representation internal variables@>=
typedef representation<T,structure,storage> rep_type;
@ @<Matrix representation methods@>=
rep_type& operator=(const rep_type& source)
{
    if(source.rows() != rows() || source.cols() != cols())
	throw error::generic("Can only assign representation to same dimension.");
    *theStorage = *(source.theStorage);
    *theStructure = *(source.theStructure);
    return *this;
}
@ Here we see why |copyfrom| can be much faster: the assignment operator for the representation
didn't involve any memory allocation. Now, the |dense| assignment operator will not allocate
memory too, and will use an optimized routine for copying elements. We can not use a single
|memcpy| because there is a possibility that the source storage has a different size of allocated
memory (because of |max_rows| and |max_cols|).
@<Dense storage methods@>=
dense<T>& operator=(const dense<T>& source)
{
    if(source.num_rows<num_rows || source.num_cols<num_cols)
	throw error::generic("Incompatible dimension in dense assignment operator.");
    for(index i=1;i<=num_cols;++i) 
	memcpy(data[i]+1,source.data[i]+1,num_rows*sizeof(element_type));
    return *this;
}
@q@>
@q Destroying @>
@** Destroying a matrix. Destroying a matrix would be very simple if it wasn't for the fact
that another matrix can be sharing the same representation. This fact makes the destruction
of a matrix to be downgraded to the ``simple'' category. When a matrix is deleted, it can
happen that there is another matrix sharing the representation. We only delete the
representation if we are the sole matrix using it. If we are not, we only update the number
of matrices sharing the representation.
@<Matrix methods@>=
@:representation}{instances@>
~matrix(void) 
{
#ifdef __MATH_INSANE_DEBUG__
cout << "[math]: ~matrix, this=" << this << ", theRepresentation=" << theRepresentation << '\n';
#endif
    if(theRepresentation && --theRepresentation->instances()==0)
	delete theRepresentation;
}
@ Deleting a representation requires a representation destructor. If you try to delete a
representation that is still shared \math{} gives you a little piece of its mind.
@<Matrix representation methods@>=
~representation(void)
{
    if(num_instances)
        throw error::generic("Deleting a referenced representation!");
    delete theStorage;
    delete theStructure;
}
@ Of course, deleting a representation requires structure and storage destructors. For the
unstructured class we rely on the default destructor, since the class is empty anyway. For
the dense storage it is convenient, like with the constructing part, to define an auxiliary
method and to call it within the destructor. In deleting the |data| vector we only have to
keep in mind that it was adjusted during creation to obey the $(1,1)$ origin default.
@<Dense storage methods@>=
@:dense}{destroy@>
~dense() @+
{@+
    destroy();@+
}
@ @<Dense storage methods@>= @!@:dense}{destroy@>
void destroy(void)
{
    if(elements) delete[] elements;
    if(data) delete[]++data;
    elements=0;
    data=0;
}
@q@>
@q Setting and getting elements @>
@** Setting and getting elements. At this point we are able to create and destroy matrices.
The next step is to allow the user to set individual elements. Since the behavior of the
|set| operation depends heavily on the matrix structure, the matrix type calls the
representation to perform the structure-storage communication protocol. The matrix must,
however, create a new representation in case the current one is shared, otherwise we will
modify other matrices too. Since this operation is useful in other situations (for example
when calling \lapack{} routines), we define a method that creates a new representation for
the matrix if necessary.
@<Matrix methods@>=
@!@:matrix}{detach@>
void detach(void)
{
    if(theRepresentation && theRepresentation->instances()>1)@:representation}{instances@>
    {
        --theRepresentation->instances();
        theRepresentation=new rep_type(*theRepresentation);
    }
}
@ We are now  in position to set an element.
@<Matrix methods@>=
@!@:matrix}{set@>
@:matrix}{detach@>
@:representation}{set@>
element_type set(index row, index col, element_type value)
{
    detach();
    return theRepresentation->set(row,col,value);
}
@ The represenation performs the handling. First it calls the structure |preprocess| method.
The structure will then modify the index accordingly so that a call to the storage |set|
method will modify the correct element. If the structure returns |false| it means the
element is not assignable.
@<Matrix representation methods@>=
@!@:representation}{set@>
@:structure}{preprocess@>@:storage}{get@>
element_type set(index row, index col, element_type value)
{
    if(theStructure->preprocess(&row,&col))
   	return theStorage->set(row,col,value);
    return theStorage->get(row,col); 
}
@ For |unstructured| structures the |preprocess| method is empty, and the |dense| |set|
method is trivial (remember the way the data is stored).
@<Unstructured structure methods@>=
@!@:unstructured}{preprocess@>
bool preprocess(index*,index*)@+ {@+ return true;@+ }
@ @<Dense storage methods@>= @!@:dense}{set@> 
element_type set(const index &row, const index &col,const element_type& value)
{@+
    return data[col][row]=value;@+
}
@ Wow! We are now able to assign elements to matrices! Next I guess you will want to
retreive them, and the most intuitive way to do that is to use the parenthesis operator. for
reasons that will become clear, we should not allow the parenthesis operator to be used for
assignment, that is, we should not allow something like $A ( i,j)=0 $ (if you insist then
yes, we can allow it, but with a huge price in performance). For this reason we do not
return a reference, but the value itself. Now to the protocol. Again, a matrix doesn't know
how to retreive the value, since the way to retreive something depends on the structure, so
it calls the representation |get| method to do the job. For consistency reasons we also
define a |get| method (to pair the |set| one).
@<Matrix methods@>=
@!@:matrix}{$()$@>
@!@:matrix}{get@>
@:representation}{get@>
const element_type operator()(const index row, const index col=1) const
{@+
    return theRepresentation->get(row,col);@+
}
const element_type get(const index row, const index col) const
{@+
    return theRepresentation->get(row,col);@+
}
@ The representation needs now to perform the communication again. First it calls the
structure |preprocess| to get the right index, and then retreive the element from the
storage. Strictly speaking we would need a postprocessing on the element value for Hermitian
matrices, but in this feature will wait a little more.
@<Matrix representation methods@>=
@!@:representation}{get@>
@:structure}{preprocess@>
@:storage}{get@>
const element_type get(index row, index col) const
{
    theStructure->preprocess(&row,&col);
    return theStorage->get(row,col);
}
@ A storage |get| method must return zero if either the column or the row indexes are zero.
In this way it is possible for a structure to force certain entries to be zero. In a
diagonal matrix, for example, the structure |preprocess| method should assign zero to the
index whenever $row\neq col$ (and also return |false|).
@<Dense storage methods@>= @!@:dense}{get@>
const element_type get(const index &row, const index &col) const
{
 if(!row || !col) return element_type(0);
    return data[col][row];
}
@ Now to the main issue. How to deal with $A(i,j)=x$? Assigning a zero element to a sparse
matrix means deleting it, so we cannot return a reference to the data. There are two
possible solutions: either we force the user to use the |set| method or we return a
structure. The first solution is cumbersome and hard to read, hence we choose the second,
and define a method that will return an |element|. An |element| works like a pointer to a
|matrix| entry. When you assign an |element| a value, it calls the matrix |set| method
accordingly.
@s matrix_type int
@<Element definition@>=
template <class matrix_type>@;
class element
{
    @<Matrix element internal variables@>@;
public:@;
    @<Matrix element methods@>@;
};
@ An |element| doesn't need to know about it's value, since, as said before, it works like a
reference to a particular index of a particular matrix. These are, then the only data an
|element| must have.
@<Matrix element internal variables@>=
matrix_type* theMatrix;
index i,j;
@ The numeric type of the element is retreived from the matrix. This type is useful
(essential!) for readability in the sequel.
@<Matrix element internal variables@>=
typedef typename matrix_type::element_type @!element_type;
@ An element is not intended to be used as a user variable, and is also not intended to be
constructed without arguments. Hence, we define the default constructor to throw an error.
An element should be constructed with all necessary arguments, which are the matrix instance
and the index to the matrix entry.
@<Matrix element methods@>=
element(void) @[:@] theMatrix(0), i(0), j(0)
{@+
    throw error::generic("Default constructor of element should not be used!");@+
}
@ @<Matrix element methods@>=
element(matrix_type* mat, const index row, const index col) @[:@]
theMatrix(mat), i(row), j(col) @+ {}
@ As aligned before, the main purpose of an element is to call the |set| method of it's
matrix when assigned a value. This will ensure the correct processing of the assignment
taking into account the matrix structure.
@<Matrix element methods@>= @!@:element}{$\K$@>
@:matrix}{set@>
inline element_type operator=(const element_type& value) const @+
{@+ return theMatrix->set(i,j,value);@+ }
@ Now we are finally able to define the matrix method to be used for readable assignments.
The method returns an element which you can assign a value in a friendly way. We name the
method |entry|, so that it makes logical sense. In a program, you would write
|A.entry(1,1)=0| or something similar. As you can see, that construction is much better, in
visual terms, than the equivalent |set| method. Depending on the compiler, however, it is
slower. But you can't always get what you want.
@<Matrix methods@>=
@!@:matrix}{entry@>
element<matrix_type> entry(const index row, const index col=1)@#
{
    return @[element<matrix_type>@](this,row,col);
}
@ By now we are able to set and get individual elements values. It is, however, useful
sometimes to be able to have direct access to the storage elements (for example, when
interfacing with another library such as \lapack). In this case, we also |detach| the matrix so
that there is no danger of messing around with shared representations.
@<Matrix representation methods@>=@!@:representation}{storg@>
storage<T>* storg(void)
{@+ return theStorage;@+}
@ @<Matrix methods@>=@!@:matrix}{storg@>
@:representation}{storg@>
storage_type* storg(void)
{
    detach();@:matrix}{detach@>
    return (theRepresentation?theRepresentation->storg():0);@:representation}{storg@>
}
@ And sometimes we may want the representation too.
@<Matrix methods@>=@!@:matrix}{rep@>
rep_type* rep(void)
{
    detach();@:matrix}{detach@>
    return theRepresentation;
}
@ Now that we have a means to get the matrix storage instance, we provide a means to have
access to the storage elements themselves. Since this method will be used only when we know
which kind of storage we have, it will differ between every storage type, and it is not
required for one.
@<Dense storage methods@>=@!@:dense}{memory@>
element_type* memory(void)@+
{@+ return elements;@+}
@q@>
@q Resizing @>
@** Resizing. We define a resize operation as one that changes the size of the matrix but leaves
the matrix elements in the same place. You can't assume anything about the value of elements that 
were not present in the original matrix. A matrix that has no representation simply creates one
with the required dimensions. Otherwise, the matrix detaches itself from others and asks for a
representation resizing.
@<Matrix methods@>=@!@:matrix}{resize@>
@:matrix}{init@>@:matrix}{detach@>@:representation}{resize@>
void resize(const index rows, const index cols)
{
    if(!theRepresentation)
    {
	init(rows,cols);
	return;
    }
    detach();
    theRepresentation->resize(rows,cols);
}
@ The protocol is as follows: first the representation calls the structure |resize| method. The
structure returns the new dimensions for the storage (or throws a |dimension| error in case the
new size is not compatible). Since the storage will throw an error if the new dimensions are
wrong, we can safely update the representation variables before resizing. Next, the representation
calls the storage |resize| method, which will take care of everything else.
@<Matrix representation methods@>=@!@:representation}{resize@>
@:structure}{resize@>@:storage}{resize@>
void resize(index rows, index cols)
{
    num_rows=rows;
    num_cols=cols;
    theStructure->resize(&rows,&cols);
    theStorage->resize(rows,cols);
}
@ @<Unstructured structure methods@>=@!@:unstructured}{resize@>
void resize(const index*,const index*) const@+{@+}
@ For the dense |resize| we will use some tricks. If the matrix can be resized
without any memory allocation, then that's what will be done. This means that in those cases the
resizing operation is very fast and that it can happen that resizing a big matrix to a small one
doesn't free any memory. If we need to allocate memory we make sure that |init| won't erase the
old data. We do that by copying it to another |dense| instance and zeroing out our data pointers.
@<Dense storage methods@>=@!@:dense}{resize@>
void resize(const index rows, const index cols);
@ @<|export|-waiting...@>=
template<class T>
void dense<T>::resize(const index rows, const index cols)
{    
    if(cols<=max_cols && rows<=max_rows)
    {
	num_rows=rows;
	num_cols=cols;
	return;
    }
    dense backup(*this);
    elements=0; data=0;
    init(rows,cols);
    for(index j=1;j<=cols && j<=backup.num_cols;++j)
    for(index i=1;i<=rows && i<=backup.num_rows;++i) data[j][i]=backup.data[j][i];
}
@q@>
@q Submatrices @>
@** Submatrices. Our next task is to provide a way to access submatrices, an operation that is
very handy in many cases. A submatrix behaves like an |element| in the sense that it does not
have any storage of its own, only a pointer to a matrix. The difference is that, while an element
is a pointer to only one matrix entry, a submatrix is a pointer to a matrix block. The internal
variables are the matrix the block refers to and the range, in the form (using \matlab{}
notation) $A(i1:i2,j1:j2)$. The basic design decision is that a submatrix, although a pointer to
a generic matrix, produces only |unstructured| matrices when algebraic operations are performed.
An example will clarify this necessity: suppose you have a submatrix of a symmetric matrix, and
that this submatrix is not symmetric. What happens when you multiply this submatrix by a number
is that you should return a |unstructured| submatrix.
@d submatrix_template(M) class @[SUBM##M@]
@<Submatrix definition@>=
template<class T>@;
class submatrix
{
    @<Submatrix internal variables@>@;
public:@;
    @<Submatrix methods@>@;
};
@ In order to be able to use submatrices from matrices (recall that the |submatrix| definition
comes after the |matrix| one in the header file) we need to predeclare the |submatrix| template.
We could have done this the other way around, but this way seems to me to be simpler.
@<Basic definitions@>=
template<class T> class submatrix;
@ In order to be able to use submatrices and matrices with the same template functions, we need
to provide a means of detecting the matrix type, element type and so on. For a submatrix, the
|matrix_type| is always |unstructured|, so we have an additional definition for the matrix the
submatrix is referring to.
@<Submatrix internal variables@>=
typedef T internal_matrix_type;
typedef typename T::element_type element_type;
typedef submatrix<T> submatrix_type;
typedef matrix<element_type,unstructured,dense> matrix_type;
@ The only data that is needed is a pointer to the matrix and the submatrix corners coordinates.
@<Submatrix internal variables@>=
internal_matrix_type *theMatrix;
index i1,i2,j1,j2;
@ A submatrix is not intended to be used as a user variable, and is also not intended to be
constructed without arguments. This ``not intented'' is strong, in the sense that no care is
currently taken to insure that the pointer a |submatrix| stores is valid in any sense. A
submatrix is intended to use immediately as a shorthand for assignments and one-line formula
references. We will adopt, for submatrices, a~$(0,0)$ internal origin standard in order to make
assignments more efficient. One of the annoying things in defining constructors is that we have
to use |const_cast| in some cases (if you are passing a submatrix as a |const| argument of some
funcion you'll need this unless you want your screen filled with ``discards const'' warnings).
@s const_cast int
@<Submatrix methods@>=
submatrix(void) @[:@] theMatrix(0)
{@+
    throw error::generic("Default constructor of submatrix sould not be used!");@+
}
submatrix(const internal_matrix_type *mat, const index row1, const index row2, const index col1=1,
	  const index col2=1)
{
    theMatrix=const_cast<internal_matrix_type*>(mat);
    i1=row1-1;
    i2=row2-1;
    j1=col1-1;
    j2=max(col2,col1)-1;
}
@ A submatrix can be assigned values in two ways: by a matrix or by a submatrix. We deal with the
former first for simplicity.
@<Submatrix methods@>=
@:matrix}{rows@>@:matrix}{cols@>@:matrix}{set@>@:matrix}{get@>
submatrix(const internal_matrix_type &mat)@+{@+ *this=mat;@+}
submatrix_type& operator=(const internal_matrix_type &mat)
{
    if(mat.rows()!=i2-i1+1 || mat.cols()!=j2-j1+1)
	throw error::dimension();
    for(index i=1;i<=mat.rows();++i)
	for(index j=1;j<=mat.cols();++j)
	    theMatrix->set(i1+i,j1+j,mat.get(i,j));
    return *this;
}
@ For the later type we need to make sure we are protected against things
like~$A(1:2,1:2)=A(2:3,2:3)$. In order to do that we will need to create a matrix from a
submatrix. The thing to be aware of is that in assigning a submatrix to a matrix we don't handle
the shared representations anymore, instead we really create a new matrix; but before doing that
we need some auxiliary methods from |submatrix| (note that the parenthesis operator obeys
the~$(1,1)$ origin default). These auxiliary methods, together with others that will be defined
later, will make a submatrix behave like a matrix for the basic operations.
@<Submatrix methods@>=
@!@:submatrix}{rows@>@!@:submatrix}{cols@>@:submatrix}{()@>
index rows(void) const@+ {@+ return i2-i1+1;@+ }@!@:submatrix}{rows@>
index cols(void) const@+ {@+ return j2-j1+1;@+ }@!@:submatrix}{cols@>
element_type operator()(const index i,const index j=1)
    const {@+return theMatrix->get(i1+i,j1+j);@+}
@ @<Matrix internal types@>=
typedef submatrix<matrix_type> submatrix_type;
@ The following |matrix| methods are auxiliary to |submatrix|, but useful in their own right.
@<Matrix methods@>=
template<class SUBM>
matrix(const submatrix<SUBM> &mat):theRepresentation(0)
{@+
    *this=mat;@+
}
@:submatrix}{rows@>@:submatrix}{cols@>@:matrix}{set@>@:submatrix}{()@>
@:matrix}{cols@>@:matrix}{rows@>@:matrix}{init@>
template<class SUBM>
matrix_type& operator=(const submatrix<SUBM> &mat)
{
    init(mat.rows(),mat.cols());
    for(index i=1;i<=rows();++i)
	for(index j=1;j<=cols();++j)
	    set(i,j,mat(i,j));
    return *this;
}
@ Now to the last type of assignment. We need to check if our matrix is the same as the one being 
assigned, and if so if there is intersection between the source and destination elements. If
that's the case we create a new matrix based on the source and copy from this matrix.
@<Submatrix methods@>=
@:submatrix}{cols@>@:submatrix}{rows@>@:matrix}{set@>@:submatrix}{()@>
submatrix(const submatrix_type& mat)@+ {@+ *this=mat;@+ }
submatrix_type& operator=(const submatrix_type &mat)
{
    if(theMatrix==mat.theMatrix && j1<=mat.j2 && j2>=mat.j1 && i1<=mat.i2 && i2>=mat.i1)@/
	*this=internal_matrix_type(mat);
    else
	for(index i=1;i<=rows();++i)
	    for(index j=1;j<=cols();++j)
		theMatrix->set(i1+i,j1+j,mat(i,j));
    return *this;
}
@ In order to make the submatrix type to resemble a matrix we define |set| and |get| methods for
it.
@<Submatrix methods@>=
@!@:submatrix}{set@>@!@:submatrix}{get@>
@:matrix}{set@>@:matrix}{get@>
void set(index row,index col,element_type value)@+{@+theMatrix->set(row+i1,col+j1,value);@+}
const element_type get(index row,index col=1)@+{@+return theMatrix->get(row+i1,col+j1);@+}
@ All we need now is a means to get a submatrix from a matrix. Since we can't overload the `|:|'
operator, there is no good way to create one except by a specialized method (instead of
overloading the parenthesis operator). We maintain some similarity with \matlab{} in the sense
that the first two arguments define the row range (and not the upper corner). The default
behavior for the last column (which is to get the value of the starting column if that is
greater) is handy sometimes. Note that the method is declared |const| even if it isn't. The
reason is the same as the one described when defining |submatrix| constructors.
@<Matrix methods@>=@!@:matrix}{subm@>
submatrix_type subm(const index row1, const index row2, const index col1=1, index col2=1) const
{
    col2=max(col2,col1);
    return submatrix_type(this,row1,row2,col1,col2);
}
@ Again we provide a |subm| method for submatrices in order to make them resemble an actual
matrix.
@<Submatrix methods@>=@!@:submatrix}{subm@>
submatrix_type subm(const index row1, const index row2,
		    const index col1=1, index col2=1) const
{
    col2=max(col2,col1);
    return theMatrix->subm(i1+row1,i1+row2,j1+col1,j1+col2);@:matrix}{subm@>
}
@q@>
@q Basic algebraic operations @>
@** Basic algebraic operations. We are now able to define the most basic algebraic operations,
such as sum, multiplication by scalars, and so on. We begin the unary operators.
The methods are straightforward, but it is not as elegant as one might want it to be because we
have to use the |set| and |get| methods: it is possible to use the already defined |entry|, but
it could be slower, and the parenthesis operator is very cumbersome to use with a pointer
(|this|). A common characteristic of the methods that follow is that the outer loop is generally
the loop on the columns. We do that because we know that the unstructured matrix stores by
column, so that it is more likely that the computer will make better use of the fast cache memory
if we access the matrix this way. Let's start with scalar multiplication:
@q *= @>
@<Matrix methods@>=
@:matrix}{cols@>@:matrix}{rows@>@:matrix}{set@>@:matrix}{get@>
matrix_type& operator @[*=@] (const T& value)
{
    for(index j=1;j<=cols();++j)@:matrix}{cols@>
        for(index i=1;i<=rows();++i)@:matrix}{rows@>
            set(i,j,value*get(i,j));@:matrix}{set@>@:matrix}{get@>
    return *this;
}
@ @<Matrix element methods@>=
element_type operator@[*=@] (const element_type& val)
{
    *this = theMatrix->get(i,j) * val;
    return value();
}
@q@>
@q /= @>
@ @<Matrix methods@>=
@:matrix}{cols@>@:matrix}{rows@>@:matrix}{set@>@:matrix}{get@>
matrix_type& operator @[/=@] (const T& value)
{
    for(index j=1;j<=cols();++j)@:matrix}{cols@>
        for(index i=1;i<=rows();++i)@:matrix}{rows@>
            set(i,j,get(i,j)/value);@:matrix}{set@>@:matrix}{get@>
    return *this;
}
@ @<Matrix element methods@>=
element_type operator/= (const element_type& val)
{
    *this = theMatrix->get(i,j)/val;
    return value();
}
@ @<Submatrix methods@>=
submatrix_type& operator/=(const element_type& value)@+
{
    for(index j=1;j<=cols();++j)
        for(index i=1;i<=rows();++i)
	    theMatrix->entry(i1+i,j1+j)/=value;
    return *this;
}
@q@>
@q += @>
@ Next we define the unary addition operator. In this case we test for dimension -- we assume
that the computational overhead in doing this is negligible when compared to the sum itself.
@<Matrix methods@>=
template<submatrix_template(A)>@;
matrix_type& operator += (const SUBMA& value)
{
    if(rows() != value.rows() || cols() != value.cols())
        throw error::dimension();@;
    
    for(index j=1;j<=cols();++j)
        for(index i=1;i<=rows();++i)
            set(i,j,get(i,j)+value(i,j));
    return *this;
}
@ @<Matrix element methods@>=
element_type operator+= (const element_type& val)
{
    *this = theMatrix->get(i,j) + val;
    return value();
}
@ @<Submatrix methods@>=
submatrix_type& operator+=(const matrix_type &value)
{
    if(value.rows() != rows() || value.cols() != cols())
	throw error::dimension();
    for(index j=1;j<=cols();++j)
	for(index i=1;i<=rows();++i)
	    theMatrix->entry(i1+i,j1+j)+=value.get(i,j);
    return *this;
}
@q@>
@q -= @>
@ @<Matrix methods@>=
template<submatrix_template(A)>@;
matrix_type& operator -= (const SUBMA& value)
{
    if(rows() != value.rows() || cols() != value.cols())
        throw error::dimension();@;
    
    for(index j=1;j<=cols();++j)
        for(index i=1;i<=rows();++i)
            set(i,j,get(i,j)-value(i,j));
    return *this;
}
@ @<Matrix element methods@>=
element_type operator-= (const element_type& val)
{
    *this = theMatrix->get(i,j) - val;
    return value();
}
@ @<Submatrix methods@>=
submatrix_type& operator-=(const matrix_type &value)
{
    if(value.rows() != rows() || value.cols() != cols())
	throw error::dimension();
    for(index j=1;j<=cols();++j)
	for(index i=1;i<=rows();++i)
	    theMatrix->entry(i1+i,j1+j)-=value.get(i,j);
    return *this;
}
@q@>
@ The transpose operator has to be defined outside, because we'll want to specialize it (for
example, for a symmetric matrix it's a void method, for a upper triangular it returns another
type of matrix, and so on).
@<Basic algebraic operations@>=
template<matrix_simple_template>@;
matrix<T,structure,storage> transpose(const matrix<T,structure,storage>& x)
{
    matrix<T,structure,storage> result(x.cols(),x.rows());
    for(index j=1;j<=x.cols();++j) for(index i=1;i<=x.rows();++i) result.set(j,i,x(i,j));
    return result;
}
template<matrix_simple_template>
matrix<T,structure,storage> transpose(const submatrix<matrix<T,structure,storage> >&x)
{
    matrix<T,structure,storage> result(x.cols(),x.rows());
    for(index j=1;j<=x.cols();++j) for(index i=1;i<=x.rows();++i) result.set(j,i,x(i,j));
    return result;
}
@q@>
@q Basic specializations and utilities @>
@** Basic specializations and utilities. We're done with the matrix type and the basic
structure and storage types. You could, using what we wrote, code anything from a matrix
multiplication procedure to a complete eigenvalue/eigenvector decomposition. Sometimes,
however, it is good to have some utilities predefined, and that's what this section is
about.

While writing code for LU decomposition I felt the need for a |swap| algorithm for matrix
elements. Since we cannot use the parenthesis operator for assignment, the STL's |swap|
function doesn't work. The reason is interesting: a swap function must store one value in a
temporary variable. The STL's |swap| assumes that this temporary variable has the same type
as the function arguments, so it does something like |element aux=A|. But, for our purposes,
this is useless -- the {\em value} of the element was not saved, only the matrix instance
and the index to the entry! Hence, the need to specialize.
@<Specializations@>=
template<class T>
inline void swap(element<T> x, element<T> y)
{
    typename T::element_type aux = x.value();
    x=y.value();
    y=aux;
}
@ @<Matrix element methods@>= @!@:element}{value@>
inline element_type value(void) const @+
{@+ return theMatrix->get(i,j);@+}
@ Of course, I needed a |swap| method because I wanted to swap rows. This operation is
useful in many other places, so I add a method that does exactly this.
@<Matrix methods@>=
@!@:matrix}{swaprows@>
void swaprows(const index i,const index k)
{
    for(index j=1;j<=cols();++j)
        swap(entry(i,j),entry(k,j));
}
@ Comparisons between matrices can be made in two ways: either we compare their elements one 
by one or we check if they share the same representation. The next method does exactly that, 
and it's useful in many contexts (there's a similar function in Lisp, for example, where we
can have shared objects).
@<Specializations@>=
template<matrix_simple_template>@;
bool same(const matrix<T,structure,storage>& x,const matrix<T,structure,storage>& y)
{
    return (x.theRepresentation == y.theRepresentation);
}
@ Next we define a new type of pair, a pair of indexes. We can use this pair for using stl's 
|map| with a |less| comparison function.
@<Specializations@>=
typedef pair<math::index,math::index> @!pair;
@ @<Specializations@>=
bool operator<(const math::pair &x, const math::pair &y);
@ @<Big definitions@>=
bool operator<(const math::pair &x, const math::pair &y)
{
    if(x.first>y.first) return false;
    if(x.first<y.first) return true;
    return(x.second<y.second);
}
@ @<Include files |math|@>=
#include <utility>
@ We will also interface this library with \lapack{} at some point. In order to define the
necessary Fortran stuff we include a file automatically generated during the library build.
The file contains some \lapack{} function prototypes and some definitions to take care of
system-dependent \lapack{} features (in some systems the function names need underscore, for
example). This file will also have the \.{LAPACK} definition built-in.
@<Include files |math|@>=
#include <math/private/fortran.h>
@ While we are at it, some sparse structures used in other programs (such as {\tt PCx}) have a
field with the number of nonzero elements in the matrix. In order to make interface easier we
define a method that returns this number.
@<Matrix methods@>=@!@:matrix}{numnonzeros@>
index numnonzeros(void) const
{
    index result=0;
    for(index j=1;j<=cols();++j)@:matrix}{cols@>
	for(index i=1;i<=rows();++i)@:matrix}{rows@>
	    if(get(i,j)) result++;@:matrix}{get@>
    return result;
}
@ The |reshape| function works exactly as in Matlab (that is, the new matrix has the same
elements taken columnwise from the original). We define it only for |unstructured| and |dense|
matrices for now. We use our knowledge about the |dense| storage to make this operation very
fast. The source matrix is overwritten with the new one.
@<Specializations@>=
template<class T>@;
matrix<T,unstructured,dense>& reshape(matrix<T,unstructured,dense>* x,index rows, index cols)
{
    if(x->rows()*x->cols() != rows*cols)@:matrix}{rows@>@:matrix}{cols@>
	throw error::dimension();
    x->storg()->init(rows,cols);@:dense}{init@>@:matrix}{storg@>
    x->rep()->rows()=rows;@:matrix}{rep@>@:representation}{rows@>
    x->rep()->cols()=cols;@:representation}{cols@>
    return *x;
}
@q@>
