@** The QR decomposition. The last two decompositions worked with square matrices, decomposing
them into matrices with some {\em structure\/} that made easy the task of solving linear systems.
The QR decomposition, on the other hand, produces~$A=QR$, where~$R$ is upper triangular and~$Q$
is {\em orthogonal,\/} that is, $Q'Q=QQ'=I$. This decomposition provides a straightforward
way to solve the problem $\min_x\norm{Ax-b}_2$, and is also the basis for one of the most
widely used algorithms for computing the SVD decomposition and the eigenvalues of a matrix.
@c /* Empty, waiting for |export|. */
@ @(qr.h@>=
#ifndef __MATH_QR__
#define __MATH_QR__ 1.0
#include<math/math.h>
#include<math/algebra.h>
namespace math
{@/
    namespace qr
    {
    @<QR prototypes@>@;
    }
}
#endif
@ The QR decomposition algorithm we will implement works by finding orthogonal matrices~$H_i$
such that $H_n\cdots H_1A=R$, where~$R$ is upper triangular. The~$H_i$ matrices are called {\sl
Householder\/} matrices, and what they do is to selectively zero out elements of a column
of~$A$. We'll see how to compute these matrices later on. A straightforward algorithm would then
be one that repeatedly computed and applied Householder transformations to the original matrix.
This is exactly the algorithm we implement here until necessity arrives for pivoting.
@<QR prototypes@>=
template<class T, template<@[class@]> class storage>@/
matrix<T,unstructured,storage>& 
decompose(matrix<T,unstructured,storage>* A)
{
    matrix<T,unstructured,dense> v(A->rows(),1);@:matrix}{rows@>
    matrix<T,unstructured,dense> w(A->cols(),1);@:matrix}{cols@>
    T beta;
    for(index j=1;j<=A->cols();++j)
    {
        @<Compute Householder vector for column |j|@>;
	@<Apply Householder transformation@>;
    }
    return *A;
}
@ A Householder reflection can in fact be represented by a single vector |v| and a scalar~$\beta$.
In fact, the definition of a Householder matrix is any~$n-{\rm by}-n$ matrix~$P$ of the form
$$I-\frac2{v^Tv}vv^T.$$ Synonyms are Householder reflection, Householder matrix. The vector~$v$
is called a {\sl Householder vector\/}. Householder matrices are easily verified to be symmetric
and orthogonal. If a vector~$x$ is multiplied by a Householder matrix, then it is reflected in
the hyperplane defined by~${\rm span}\set{v}^\perp$. In particular, suppose we have a vector~$x$
and 
want to zero out all but the first component, that is, we want~$Px$ to be a multiple of~$e_1$.
After some algebra we can see that $$v=x\pm\|x\|_2e_1$$ gives the desired transformation. The
following piece of code will compute a Householder vector such that~$v(1)=1$, $\beta=2/v^Tv$,
and~$Px=\|x\|_2e_1$. The normalization of the first entry of the Householder vector is desirable
becaise we can store it with one less component (which enables us to store them directly
in the~$A$ matrix together with~$R$). Also, we don't use the above formula because of numerical
problems. We use instead $$v_1=x_1-\|x\|_2=\frac{-(x_2^2+\cdots+x_n^2)}{x_1+\|x\|_2}$$ which is
numerically more stable when~$x_1>0$.
@<Compute Householder vector...@>=
v.resize(A->rows()-j+1,1);
v.entry(1)=T(0.0);
v.subm(2,v.rows())=A->subm(j+1,A.rows(),j,j);
T sigma=dot(v,v);
v.entry(1)=T(1.0);
if(sigma==0) beta=0;
else {
    T x = A->get(j,j);
    T mu=sqrt(sigma+x*x);
    v.entry(1)=(x<=0 ? x-mu : -sigma/(x+mu));
    T v1 = v(1);
    beta=2*v1*v1/(sigma+v1*v1);
    v/=v1;
}
@ In order to apply the Householder transformation we need to be careful. We actually don't need
any matrix-matrix products if we realize that $$PA=(I-\beta vv^T)A=A-\beta vv^TA,$$ and
$vv^TAv^T=v(A^Tv)^T,$ which consists only of matrix-vector and vector-vector products. Thus, the
Householder update requires only a vector-matrix multiplication and an outer product update. In
our case, we can also make use of some facts: we know that~$v(1)=1$ and that a block of the~$A$
matrix is already zeroed out.
@<Apply Householder...@>=
submatrix<matrix<T,unstructured,storage> > Ablock(A,j,A->rows(),j,A->cols());
w=atxmul(Ablock,v,&w);
(void) outerp_update(&Ablock,-beta,v,w);
if(j<A->rows()) A->subm(j+1,A->rows(),j,j)=v.subm(2,v.rows());
@* Solving linear equations. We now are in position to solve two important problems. The
first is the so-called {\sl least-squares problem\/}, where we find~$x$ that solves $\min
\norm{Ax-b}_2$, where $A\in\reals[m][n]$ and~$m>n$. In this case the system is {\sl
overdetermined\/}, and an exact solution to~$Ax=b$ may not exist.

The second problem occurs when~$A\in\reals[m][n]$ and~$n>m$. The system~$Ax=b$ either has an
infinite number of solutios or none. If it does have one, we compute the {\sl minimum norm\/}
solution.
@<QR prototypes@>=
template<class T, template<@[class@]> class storage>@/
matrix<T,unstructured,storage>& 
solve(matrix<T,unstructured,storage>* A, matrix<T,unstructured,dense>* B)
{
    index l=B->cols();
    index m=A->rows();
    index n=A->cols();
    matrix<T,unstructured,dense> w(l,1);
    if(m>=n)
    { @<Solve least squares problem@>; }
    else
    { @<Solve minimum norm problem@>; }
    return *B;
}
@* Least squares. The least squares problem is solved via a direct matrix decomposition. The
solution matrix is 
guaranteed to be smaller than~$B$ and has the same number of columns, so we overwrite~$B$ with
the solution and resize it accordingly before returning.
@<Solve least squares problem@>=
(void) decompose(A);
@<Solve $Qy=b$@>;
@<Solve $Rx=y$@>;
B->resize(n,l);
@ To solve~$Qy=b$ we have to remember that~$Q$ is orthogonal, so all we have to do is to
compute~$y=Q^Tb$.
@<Solve $Qy=b$@>=
matrix<T,unstructured,storage> v(m,1);
v.entry(1)=T(1);
for(index j=1;j<=n;++j)
{
    v.resize(m-j+1,1);
    v.subm(2,v.rows())=A->subm(j+1,m,j,j);
    submatrix<matrix<T,unstructured,dense> > Bblock(B,j,m,1,l);
    w=atxmul(Bblock,v,&w);
    outerp_update(&Bblock,-2/dot(v,v),v,w);
}
@ The last part is just back substitution with an upper triangular matrix.
@<Solve $Rx=y$@>=
for(index k=1;k<=l;++k)
    for(index i=n;i>=1;--i)
    {
        T inner=0;
	for(index j=i+1;j<=n;++j) inner += A->get(i,j)*B->get(j,k);
	if(!finite(B->entry(i,k)=(B->get(i,k)-inner)/A->get(i,i)))
	    throw error::rankdefficient();
    }
@* Minimum norm. The minimum norm problem is solved through a decomposition of the transposed
matrix. If~$Ax=b$, then we take the decomposition of~$A^T$ and solve the system~$(QR)^Tx=b$. The
matrix dimensions now will be $Q\in\reals[n][n]$ and~$R\in\reals[n][m]$. The matrix we'll work
with is |At|, which is in~$\reals[n][m]$.
@<Solve minimum norm problem@>=
matrix<T,unstructured,storage> At=transpose(*A);
(void) decompose(&At);
@<Solve $R^Ty=b$@>;
@<Solve $Q^Tx=y$@>;
@ Our system consists of~$R^TQ^Tx=b$. We first solve~$R^Ty=b$. This equation actually translates
to~$\bmatrix{R_1 & 0}\bmatrix{y_1\;;\;y_2}=b$, so we have no means of determining~$y_2$. This
poses no problem since we can determine~$y_1$ and that's all we need. We'll overwrite~$y_1$
on~|B|.
@<Solve $R^Ty=b$@>=
for(index k=1;k<=l;++k)
    for(index i=1;i<=m;++i)
    {
        T inner=0;
        for(index j=1;j<i;++j)
	    inner += At(j,i)*B->get(j,k);
	if(!finite(B->entry(i,k)=(B->get(i,k)-inner)/At.(i,i)))
	    throw error::rankdefficient();
    }
@ Now we use the fact that~$Q$ is orthogonal, so that~$Q^Tx=y$ translates to~$x=Qy$. The only
problem is that we don't have~$y$, just~$y_1$, so that we can't use the |outerp_update| function
here because the matrix dimensions wouldn't match.
@<Solve $Q^Tx=y$@>=
B->resize(n,l);
B->subm(m+1,n,1,l)=matrix<T,unstructured,storage>(n-m,l);
matrix<T,unstructured,storage> v(n,1);
v.entry(1)=T(1);
for(index k=m;k>=1;--k)
{
    index nrows=n-k+1;
    v.resize(nrows,1);
    v.subm(2,nrows)=At.subm(k+1,n,k,k);
    submatrix<matrix<T,unstructured,dense> > Bblock(B,k,n,1,l);
    w=atxmul(Bblock,v,&w);
    outerp_update(&Bblock,-2/dot(v,v),v,w);
}
