@* Sequential unconstrained minimization. % -*- mode: cweb; mode: fold -*- With a minimization
function defined, we are ready to tackle our main problem, namely $$\minimize{f_0(x)}{f_i(x)\prec
0,& i=1,\ldots,p\cr g_i(x)=0, & i=1,\ldots,q.}$$ The method used to solve this problem is to
minimize, for increasing values of~$t$, the functional~$$\mu f_0(x)+\sum\phi_i(x),$$
where~$\phi_i$ are barrier functions related to the inequality constraints. The equality
constraints are taken care by the search direction function --- the method doesn't do anything
to enforce them. The algorithms that work this way are called {\em Sequential Unconstrained
Minimization Techniques,\/} or SUMT for brevity.
@c /* Empty, waiting for |export| */
@ @(sumt.h@>=
#ifndef __MATH_SUMT__
#define __MATH_SUMT__
#include <vector>
#include <math/fmin.h>
#include <math/functional/linear.h>
#include <math/functional/gaxpy.h>
#include <math/barrier/log.h>
#include <math/linesearchbase.h>
namespace math
{
    @<SUMT functions@>;
}
#endif
@ The arguments for the function are as follows: first we have the objective functional and a list
of barrier functions. If the objective functional is zero, a feasibility problem will be solved.
We have to enforce that the list is actually of barrier functions because we'll need information
about dual variables in order to determine that our solution is precise enough, to detect
infeasibility and so on. Next, the user provides an initial point, and line minimization and
search direction algorithms. The last parameters are the absolute and relative precisions,
followed by a pointer to a function that gets as arguments the current phase, the value of the
objective function (primal) and the dual slack.
@<SUMT functions@>=
template<class T>@/
void
sumt(functional::base<T>* obj,
     vector<barrier::base<T>*>& barrier_functions,
     matrix<T,unstructured,dense>& x,
     linesearch::base<T>* line,
     searchdir::base<T>* dir,
     T abstol, T reltol,
     void(*disp)(int,double,double)=0)
{
#ifdef __MATH_DEBUG__
cout << "[math]: sumt begins.\n";
#endif
    T mu;
    functional::linear<T> f;
    typedef typename vector<barrier::base<T>*>::iterator sumt_iterator;
    @<Build unconstrained objective function@>;
    bool initial_point_is_feasible=true;
    for(sumt_iterator i=barrier_functions.begin();
	    i!=barrier_functions.end() && initial_point_is_feasible;++i)
	try { (*i)->eval(x);@:functional}{eval@> }
        catch(error::domain) { initial_point_is_feasible=false; }
    if(!initial_point_is_feasible)
    {	@<SUMT Phase one@>; }
    @<SUMT Phase two@>;
#ifdef __MATH_DEBUG__
cout << "[math]: sumt ends.\n";
#endif
}
@ Before doing anything we build the functional we'll minimize. We use the |linear| functional,
which has provision for weighting.
@<Build unconstrained objective function@>=
f.add(obj);
for(sumt_iterator i=barrier_functions.begin();i!=barrier_functions.end();++i)
    f.add(*i);
@q Phase I @>
@ At this point we know something is wrong with the initial point. The problem is obviously not
feasible here, but there can be two reasons: one is that one of the constraints is positive, so
the barrier function throws a |domain| error. But it can happen that the functional called by
the barrier function throws a |domain| error too. Consider that the constraint is~$\sqrt x>1$
and that the initial point is~$-1$. A log barrier function would call~$\sqrt{-1}$, which would
throw a |domain| error. The first type of infeasibility is easy to deal with through a Phase~I
approach. The other is hard in this phase, so we assume the user will provide good enough a initial
point. If not, the next piece of code will end up throwing |domain|, which will flag the problem.
@ Ok. Now that we don't have a feasible point, we modify the problem so it becomes feasible. The
standard way to do that is to modify all constraints of the form~$f(x)<0$ to~$f(x)<t$, where~$t$
is an auxiliary variable. The |addt| function of barrier functions update the constraint and
returns the minimum value of~$t$ necessary in order for the constraint to be feasible with the~$x$
provided. At the end of the loop we know that |aux_var| will be positive (the problem was
infeasible), so we can multiply it by two in order to obtain an interior point.
@<SUMT Phase one@>=
T aux_var=0;
for(sumt_iterator i=barrier_functions.begin(); i!=barrier_functions.end();++i)
    aux_var=max(aux_var,(*i)->addt(x));@:barrier}{addt@>
aux_var=aux_var+aux_var;
@ By now we have an interior feasible point for the augmented system. What we have to do is to
build the initial point vector accordingly and to build the unconstrained function to be minimized.
@<SUMT Phase one@>=
x.resize(x.rows()+1,1);
x.entry(x.rows())=aux_var;
matrix<T,unstructured,dense> phasei_cost(x.rows(),1);
phasei_cost.entry(x.rows())=1.0;
functional::gaxpy<T,unstructured,dense> phasei(phasei_cost,0.0);
f.set_term(0,&phasei);
@ We now try to find a feasible point for the original problem.  We don't have to go all the way,
just enough so that the final value of~$t$ is not too close to zero, otherwise the initial point
for phase~II will be barely feasible and numerical problems could arise.  If we find the
global optimum and~$t\geq 0$, then the problem is infeasible. We also compute the Lagrange
dual function in order to test for infeasibility. The value of the Lagrange dual
function depends on the barrier functions being used.
@<SUMT Phase one@>=
mu=1;
T cost=2*abstol;
T lagrange=0;
while(cost-lagrange>=abstol && x(x.rows())>-0.1)
{
    f.set_weight(0,mu);
    try { x=fmin(&f,x,line,dir,abstol,reltol,0.0); }
    catch(error::maxiterations e) {}
    cost = x(x.rows());
    lagrange = cost;
    for(int i=1;i!=f.size();++i) lagrange += 
        static_cast<barrier::base<T>*>(f.get_term(i))->dual_times_f(x)/mu;
    if(disp) disp(1,cost,cost-lagrange);
    if(lagrange>=0) throw error::infeasible();
    mu=mu*50;
}
if(x(x.rows())>=0) throw error::infeasible();
x.resize(x.rows()-1,1);
for(sumt_iterator i=barrier_functions.begin();
    i!=barrier_functions.end();++i)
    (*i)->delt();@:barrier}{delt@>
@q@>
@q Phase II @>
@ Ready we are for phase II. We have to restore the original cost function that was overwritten
in phase I. After that, we procceed almost exactly as in phase I.
@<SUMT Phase two@>=
if(!obj) return; /* It was a feasibility problem. */
f.set_term(0,obj);
bool stop=false;
mu=1;
while(!stop)
{
    f.set_weight(0,mu);
    x=fmin(&f,x,line,dir,abstol,reltol,0.0);
    T cost=obj->eval(x);
    T slack=0;
    for(int i=1;i!=f.size();++i) slack -= 
        static_cast<barrier::base<T>*>(f.get_term(i))->dual_times_f(x)/mu;
    if(disp) disp(2,cost,slack);
    mu=mu*50;
    stop = true;
#ifdef __MATH_DEBUG
cout << "[math]:";
cout<<"slack="<<slack<<", cost="<<cost<<", abstol="<<abstol<<", reltol="<<reltol<<'\n';
#endif
    if(abstol) stop = (slack <= abstol);
    if(reltol && cost && stop) stop = (fabs(slack/cost) <= reltol);
}
@q@>
